# coding:utf-8
import colorlog  # 控制台日志输入颜色
import logging
log_colors_config = {
    'DEBUG': 'cyan',
    'INFO': 'green',
    'WARNING': 'yellow',
    'ERROR': 'red',
    'CRITICAL': 'purple',
}



def get_logger(stage,level):

    logger_obj = logging.getLogger()
    if level==1:
        logger_obj.setLevel(logging.DEBUG)
    elif level==2:
        logger_obj.setLevel(logging.INFO)
    elif level==3:
        logger_obj.setLevel(logging.WARNING)
    elif level==4:
        logger_obj.setLevel(logging.ERROR)
    else:
        logger_obj.setLevel(logging.CRITICAL)

    if stage=="pro":

        ch = logging.StreamHandler()                           #创建一个屏幕输出流；
        ch.setLevel(logging.INFO)                           #定义屏幕输出流的告警级别；
        formater = logging.Formatter('%(asctime)s-[%(filename)s:%(lineno)d] [%(module)s:%(funcName)s] [%(levelname)s]- %(message)s')  # 自定义日志的输出格式，这个格式可以被文件输出流和屏幕输出流调用；
        ch.setFormatter(formater)
        logger_obj.addHandler(ch)

    else:

        ch = logging.StreamHandler()  # 创建一个屏幕输出流；
        ch.setLevel(logging.DEBUG)  # 定义屏幕输出流的告警级别；
        formater =colorlog.ColoredFormatter('%(log_color)s[%(asctime)s] [%(filename)s:%(lineno)d] [%(module)s:%(funcName)s] [%(levelname)s]- %(message)s',log_colors=log_colors_config)  # 日志输出格式
        ch.setFormatter(formater)
        logger_obj.addHandler(ch)

    return logger_obj                                      #将我们创建好的logger对象返回

stage="pro"
level=2
logger_obj=get_logger(stage,level)







from huihe.device.constant import SWITCH_OEM_MODEL,LIGHT_OEM_MODEL,HUMIDIFIER_OEM_MODEL,IRDEVICE_OEM_MODEL
import os
import json
from huihe.device.log import logger_obj
import datetime

irdata_path="irdata_id.json"
class HuiheDevice(object):
    def __init__(self,hass, data, api):
        self.api = api
        self.data = data
        self.oem_model = data.get('oem_model')
        self.irdata_path = os.path.join(
            hass.config.config_dir, irdata_path)
        connection_status=data.get('connection_status')
        if connection_status=="Offline":
            self.connection_status= False
        else:
            self.connection_status = True
        if data.get('oem_model') in IRDEVICE_OEM_MODEL:
            self.dev_type = data.get('dev_type')
            self.obj_id = data.get('subdevice_id')
            self.subdevice_typeID = data.get('subdevice_typeID')
            self.dsn = data.get('dsn')
            self.obj_type = "IR"
            self.obj_name = data.get('subdevice_name')
            self.irdata_id = data.get('irdata_id')
            irdata_id = self.irdata_id
            if self.data["dev_type"] == "ac":
                deviceId = self.dsn
                subdeviceName = self.data.get('subdevice_name')
                acData, code = self.api.getACStateInfo(deviceId, subdeviceName)
                if acData.get("state") == 0:
                    self.data["state"] = True
                else:
                    self.data["state"] = False
                self.data["mode_list"] = acData.get("mode_list")
                irdatas,code=self.api.getIrCode(irdata_id)
                if code == 200 or code == 201:
                    with open(self.irdata_path, 'w') as file_obj:json.dump(irdatas, file_obj)
                else:
                    pass
            else:
                self.data["state"] =None
                irdatas,code = self.api.getIrCode(irdata_id)
                if code == 200 or code == 201:
                    with open(self.irdata_path, 'w') as file_obj:
                        json.dump(irdatas, file_obj)
                    pass
                else:
                    pass

        else:

            if data.get('oem_model') in SWITCH_OEM_MODEL:
                self.obj_type = "switch"
            elif data.get('oem_model') in LIGHT_OEM_MODEL:
                self.obj_type = "light"
            elif data.get('oem_model') in HUMIDIFIER_OEM_MODEL:
                self.obj_type = "climate"
            else:
                pass

            self.obj_name = data.get('product_name')
            self.obj_id = data.get('dsn')
            self.dev_type = data.get('oem_model')
            propertyData = self.api.retrieves_properties(self.obj_id)

            for propertyData in propertyData:
                if "state" in propertyData:
                    self.data["state"] = propertyData.get('state')
                elif "brightness" in propertyData:
                    self.data["brightness"] = propertyData.get('brightness')
                elif "color_temp" in propertyData:
                    self.data["color_temp"] = propertyData.get('color_temp')
                elif "target_humidity" in propertyData:
                    self.data["target_humidity"] = propertyData.get('target_humidity')
                    self.data["humidity"] = propertyData.get('target_humidity')
                elif "current_humidity" in propertyData:
                    self.data["current_humidity"] = propertyData.get('current_humidity')
                elif "workmode" in propertyData:
                    self.data["workmode"] = propertyData.get('workmode')
                elif "mist" in propertyData:
                    self.data["mist"] = propertyData.get('mist')
                else:
                    pass


    def name(self):
        return self.obj_name

    def state(self):
        return self.data.get("state")

    def device_type(self):
        return self.dev_type

    def get_oem_model(self):
        return  self.data.get("oem_model")

    def object_id(self):
        oem = self.oem_model.replace('-', '')
        ID = 'huihe#{}#{}'.format(oem,self.obj_id)
        return ID

    def available(self):
        return True

        # return self.connection_status



    def update(self):
        """Avoid get cache value after control."""

        logger_obj.debug("pass update obj_id : %s", self.obj_id)
        deviceId = self.obj_id
        # from .iFutureHomeapi import iFutureHomeApi
        # huihe = iFutureHomeApi()
        # product_name, connection_status = huihe.get_single_device(deviceId)
        # if product_name==None:
        #     pass
        # else:
        #     self.obj_name = product_name
        # if connection_status=="Offline":
        #     self.connection_status= False
        # else:
        #     self.connection_status = True
        # print(self.obj_id,self.connection_status)

        if self.data.get('oem_model') in IRDEVICE_OEM_MODEL:
            if self.data["dev_type"] == "ac":
                logger_obj.debug("update ac")
                deviceId = self.dsn
                from ifuturehome.huihe.iFutureHomeapi import iFutureHomeApi
                huihe = iFutureHomeApi()
                body, code = huihe.get_IR_device(deviceId)
                for device in body:
                    if device["subdevice_id"]==self.obj_id:
                        self.obj_name=device["subdevice_name"]
                    else:
                        pass

                subdeviceName =self.obj_name
                acData, code = self.api.getACStateInfo(deviceId, subdeviceName)
                logger_obj.debug("acData：  %s ",acData)
                if acData.get("state") == 0:
                    self.data["state"] = True
                else:
                    self.data["state"] = False
                self.data["mode_list"] = acData.get("mode_list")
            else:
                deviceId = self.dsn
                from ifuturehome.huihe.iFutureHomeapi import iFutureHomeApi
                huihe = iFutureHomeApi()
                body, code = huihe.get_IR_device(deviceId)
                for device in body:
                    if device["subdevice_id"] == self.obj_id:
                        self.obj_name = device["subdevice_name"]
                    else:
                        pass

        else:
            # pass
            # deviceId = self.obj_id



            nowTime = datetime.datetime.now()
            logger_obj.debug("beging update time is：  %s", str(nowTime))
            propertyData = self.api.retrieves_properties(deviceId)
            for propertyData in propertyData:
                if "state" in propertyData:
                    self.data["state"] = propertyData.get('state')
                elif "brightness" in propertyData:
                    self.data["brightness"] = propertyData.get('brightness')
                elif "color_temp" in propertyData:
                    self.data["color_temp"] = propertyData.get('color_temp')
                elif "target_humidity" in propertyData:
                    self.data["target_humidity"] = propertyData.get('target_humidity')
                    self.data["humidity"] = propertyData.get('target_humidity')
                elif "current_humidity" in propertyData:
                    self.data["current_humidity"] = propertyData.get('current_humidity')
                elif "workmode" in propertyData:
                    self.data["workmode"] = propertyData.get('workmode')
                elif "mist" in propertyData:
                    self.data["mist"] = propertyData.get('mist')
                else:
                    pass
        nowTime = datetime.datetime.now()
        return True
















from huihe.device.huiHeLight import HuiHeLight
from huihe.device.huiHeSwitch import HuiHeSwitch
from huihe.device.HuiHeIR import HuiHeIR
from huihe.device.HuiHeClimate import HuiHeClimate
from huihe.device.constant import SWITCH_OEM_MODEL,LIGHT_OEM_MODEL,HUMIDIFIER_OEM_MODEL,IRDEVICE_OEM_MODEL

def get_huihe_device(hass,data, api):
    dev_type = data["device"]["oem_model"]
    devices = []
    data=data["device"]
    if dev_type in LIGHT_OEM_MODEL:
        devices.append(HuiHeLight(hass,data, api))
    elif dev_type in HUMIDIFIER_OEM_MODEL:
        devices.append(HuiHeClimate(hass,data, api))
    elif dev_type in IRDEVICE_OEM_MODEL:
        devis_list = HuiHeIR(hass,data, api)
        for device in devis_list:
            devices.append(device)
    # elif dev_type == 'fan':
    #     devices.append(TuyaFanDevice(data, api))
    # elif dev_type == 'cover':
    #     devices.append(TuyaCover(data, api))
    # elif dev_type == 'lock':
    #     devices.append(TuyaLock(data, api))
    elif dev_type in SWITCH_OEM_MODEL:
        devices.append(HuiHeSwitch(hass,data, api))
    return devices






from huihe.device.base import HuiheDevice
from huihe.device.constant import HUMIDIFIER_OEM_MODEL,IRDEVICE_OEM_MODEL
from huihe.device.climateConst import (
    HVAC_MODE_COOL, HVAC_MODE_DRY, HVAC_MODE_FAN_ONLY,
    HVAC_MODE_HEAT, HVAC_MODE_AUTO, HVAC_MODE_OFF)
from  huihe.device import const
import logging
from  huihe.device.constant import TYPE_HUMI_023
import json
_LOGGER = logging.getLogger(__name__)
from  huihe.device.log import logger_obj



class HuiHeClimate(HuiheDevice):


    def temperature_unit(self):
        return const.TEMP_CELSIUS


    def current_humidity(self):
        if self.data.get('current_humidity') is None:
            return None
        return self.data.get('current_humidity')


    def target_humidity(self):
        if self.data.get('target_humidity') is None:
            return None
        return self.data.get('target_humidity')


    def target_humidity_step(self):
        if self.data.get('target_humidity') is None:
            return None
        return 5


    def preset_mode(self):
        """Return hvac operation ie. heat, cool mode.
        Need to be one of HVAC_MODE_*."""

        if self.data.get('state') is None:
            return None
        else:
            if self.data.get('state')==0:
                return "Off"

            elif self.data.get('workmode')==2:
                return "Sleep"
            elif self.data.get('workmode')==1:
                return "Night Light"
            elif self.data.get('workmode')==0:
                if  self.data.get('mist')==3:
                    return "High Mist"
                elif self.data.get('mist')==2:
                    return "Medium Mist"
                elif self.data.get('mist')==1:
                    return "Low Mist"
                else:
                    return "Standard"


    def set_preset_mode(self, preset_mode):
        """Set new target preset mode."""
        if preset_mode=="Standard":
            Attributes="workmode"
            value = 0
        elif preset_mode=="Night Light":
            Attributes="workmode"
            value = 1
        elif preset_mode=="Sleep":
            Attributes = "workmode"
            value =2
        elif preset_mode=="High Mist":
            Attributes = "mist"
            value =3
        elif preset_mode=="Medium Mist":
            Attributes = "mist"
            value =2
        elif preset_mode=="Low Mist":
            Attributes = "mist"
            value =1

        self.api.device_control(self.obj_id,Attributes, value)


    def current_operation(self):
        current_operation = 'auto'
        if "mode_list" in self.data and "modelType" in self.data["mode_list"]:

            modelType=self.data["mode_list"].get('modelType')
            if modelType==0:
                current_operation='cool'
            elif modelType==1:
                current_operation = 'heat'
            elif modelType==2:
                current_operation = 'auto'
            elif modelType==3:
                current_operation = 'fan_only'
            elif modelType==4:
                current_operation = 'dry'

        return current_operation


    def operation_list(self):
        HVAC_MAP = {
            HVAC_MODE_HEAT: 'heat',
            HVAC_MODE_AUTO: 'auto',
            HVAC_MODE_DRY: 'dry',
            HVAC_MODE_FAN_ONLY: 'fan',
            HVAC_MODE_COOL: 'cool',
            HVAC_MODE_OFF: 'off'
        }

        return HVAC_MAP


    def current_temperature(self):
        current_temperature=26
        if "mode_list" in self.data and "curTmp" in self.data["mode_list"]:
            current_temperature = self.data["mode_list"].get('curTmp')
            if current_temperature is None:
                return None
        return current_temperature


    def target_temperature(self):
        if self.data["oem_model"] in HUMIDIFIER_OEM_MODEL:
            return None
        else:
            target_temperature=26
            if "mode_list" in self.data and "curTmp" in self.data["mode_list"]:
                target_temperature = self.data["mode_list"].get('curTmp')
                if target_temperature is None:
                    return None
        return target_temperature


    def target_temperature_step(self):
        return 1


    def current_fan_mode(self):
        """Return the fan setting."""
        if self.data["oem_model"] in HUMIDIFIER_OEM_MODEL:
            return None
        else:
            fan_speed = 'auto'
            if "mode_list" in self.data and "curWindSpeed" in self.data["mode_list"]:
                fan_speed = self.data["mode_list"].get('curWindSpeed')
                if fan_speed is None:
                    return None
                if fan_speed == '0':
                    return 'auto'
                elif fan_speed == '1':
                    return 'low'
                elif fan_speed == '2':
                    return 'medium'
                elif fan_speed == '3':
                    return 'high'

        return fan_speed


    def fan_modes(self):
        """Return the list of available fan modes."""
        if self.data["oem_model"] in HUMIDIFIER_OEM_MODEL:
            return None
        else:
            fan_list = "auto"
            if "mode_list" in self.data and "windSpeedList" in self.data["mode_list"]:
                fan_speed_list = self.data["mode_list"].get('windSpeedList')
                fan_list=[]
                if fan_speed_list is None:
                    pass
                if fan_speed_list == '0':
                    fan_list.append('auto')
                elif fan_speed_list == '1':
                    fan_list.append('low')
                elif fan_speed_list == '2':
                    fan_list.append('medium')
                elif fan_speed_list == '3':
                    fan_list.append('high')

        return fan_list


    def min_temp(self):
        if self.data["oem_model"] in HUMIDIFIER_OEM_MODEL:
            return None
        else:
            min_temper = 16
            if "mode_list" in self.data and "lowTmp" in self.data["mode_list"]:
                min_temper = self.data["mode_list"].get('lowTmp')
                if min_temper is None:
                    return None
                if min_temper==0:
                    return 17

        return min_temper


    def max_temp(self):
        if self.data["oem_model"] in HUMIDIFIER_OEM_MODEL:
            return None
        else:
            max_temper = 16
            if "mode_list" in self.data and "highTmp" in self.data["mode_list"]:
                max_temper = self.data["mode_list"].get('highTmp')
                if max_temper is None:
                    return None
                if max_temper==0:
                    return 30

        return max_temper


    def min_humidity(self):
        if self.data.get('target_humidity') is None:
            return None
        return 30


    def max_humidity(self):
        if self.data.get('target_humidity') is None:
            return None
        return 95


    def set_temperature(self, temperature):
        """Set new target temperature."""
        with open(self.irdata_path) as f_obj:
            irdatas = json.load(f_obj)

        if self.data["state"] == True:
            irCodes = irdatas["keys"]
            fan_speed = self.data["mode_list"].get('curWindSpeed')
            hvac_mode = self.data["mode_list"].get('modelType')
            acValue = 'M' + str(hvac_mode) + '_T' + str(int(temperature)) + '_S' + str(fan_speed)
            for code in irCodes:
                if code["fkey"] == acValue:
                    endpointId = self.data.get('dsn')
                    self.api.ir_control(endpointId, "ifracmd_to_dev", code)
                else:
                    pass
            self.data["mode_list"]['curTmp']=temperature
        else:
            logger_obj.warning("CLIMATE IS " + str(self.huihe.state()) + ", can not set_temperature")


    def set_timer (self, command,dev_type):
        if dev_type == TYPE_HUMI_023:
            if command in[2,4,6,8,10]:
                self.api.device_control(self.obj_id, 'timer', command)
                _LOGGER.debug("erro 023 humidifer timer , timer is " + command)
            else:
                pass

        else:
            self.api.device_control(self.obj_id, 'timer', command)


    def set_humidity(self, humidity):
        """Set new target humidity."""
        self.api.device_control(self.obj_id, 'humi',humidity)


    def set_fan_mode(self, fan_mode):
        """Set new target fan mode."""
        print("fan_mode", fan_mode)
        value=""
        irdata_id = self.data.get('irdata_id')
        with open(self.irdata_path) as f_obj:
            irdatas = json.load(f_obj)
        if fan_mode == 'auto':
            value = 0
        elif fan_mode == 'low':
            value = 1
        elif fan_mode == 'medium':
            value = 2
        elif fan_mode == 'high':
            value = 3

        if self.data["state"] == True:
            irCodes = irdatas["keys"]
            hvac_mode = self.data["mode_list"].get('modelType')
            current_temperature = self.data["mode_list"].get('curTmp')
            acValue = 'M' + str(hvac_mode) + '_T' +  str(int(current_temperature)) + '_S' + str(value)
            for code in irCodes:
                if code["fkey"] == acValue:
                    endpointId = self.data.get('dsn')
                    self.api.ir_control(endpointId, "ifracmd_to_dev", code)
                else:
                    pass
            self.data["mode_list"]['curWindSpeed']=value
        else:
            logger_obj.warning("CLIMATE IS " + str(self.huihe.state()) + ",set_fan_mode IS " + str(fan_mode))


    def set_hvac_mode(self, hvac_mode):
        """Set new target operation mode."""

        endpointId = self.data.get('dsn')
        with open(self.irdata_path) as f_obj:
            irdatas = json.load(f_obj)
        if hvac_mode=='off':

            irCodes = irdatas["keys"]
            for code in irCodes:
                if code["fkey"] == "power_off":
                    self.api.ir_control(endpointId, "ifracmd_to_dev", code)
                else:
                    pass
        else:
            current_temperature = self.data["mode_list"].get('curTmp')
            fan_speed = self.data["mode_list"].get('curWindSpeed')
            if hvac_mode=='cool':
                modeType=0
                acValue = 'M' + str(modeType) + '_T' +  str(int(current_temperature)) + '_S' + str(fan_speed)
            elif hvac_mode=='heat':
                modeType = 1
                acValue = 'M' + str(modeType) + '_T' +  str(int(current_temperature))+ '_S' + str(fan_speed)
            elif hvac_mode=='auto':
                modeType = 2
                acValue = 'M' + str(modeType) + '_T' +  str(int(current_temperature))
            elif hvac_mode=='fan_only':
                modeType = 3
                acValue = 'M' + str(modeType) + '_S' + str(fan_speed)
            elif hvac_mode== 'dry':
                modeType =4
                acValue = 'M' + str(modeType) + '_T' +  str(int(current_temperature))

            irCodes = irdatas["keys"]
            for code in irCodes:
                if code["fkey"] == acValue:
                    endpointId = self.data.get('dsn')
                    self.api.ir_control(endpointId, "ifracmd_to_dev", code)
                else:
                    pass

        self.data["mode_list"]['modelType'] = modeType


    def support_target_temperature(self):
        if self.data["oem_model"] in HUMIDIFIER_OEM_MODEL:
            return None
        else:
            if self.data.get('mode_list') is not None:
                if self.data["mode_list"].get('curTmp') is not None:
                    return True
                else:
                    return False
            else:
                return False


    def support_target_temperature_range(self):
        if self.data["oem_model"] in HUMIDIFIER_OEM_MODEL:
            return None
        else:
            return True
            # if self.data["mode_list"].get('modelType') is not None:
            #     return True
            # else:
            #     return False


    def support_wind_speed(self):
        if self.data["oem_model"] in HUMIDIFIER_OEM_MODEL:
            return None
        else:
            if self.data["mode_list"].get('windSpeedCanControl')== True:
                return True
            else:
                return False


    def support_humidity(self):
        if self.data.get('target_humidity') is not None:
            return True
        else:
            return False


    def support_preset_modes(self):
        if self.data["oem_model"] in HUMIDIFIER_OEM_MODEL:
            if self.data.get('workmode') is not None:
                return True
            else:
                return False

        else:
            return None


    def turn_on(self):
        if self.data.get('oem_model') in HUMIDIFIER_OEM_MODEL :
            self.api.device_control(self.obj_id,"switch", '1')
        elif self.data.get('oem_model') in IRDEVICE_OEM_MODEL:

            with open(self.irdata_path) as f_obj:
                irdatas = json.load(f_obj)
            irCodes = irdatas["keys"]
            current_temperature = self.data["mode_list"].get('curTmp')
            fan_speed = self.data["mode_list"].get('curWindSpeed')
            hvac_mode=self.data["mode_list"].get('modelType')

            acValue = 'M' + str(hvac_mode) + '_T' + str(int(current_temperature)) + '_S' + str(fan_speed)
            for code in irCodes:
                if code["fkey"] == acValue:
                    endpointId = self.data.get('dsn')
                    self.api.ir_control(endpointId, "ifracmd_to_dev", code)
                else:
                    pass

            self.data["state"] =True
        else:
            self.api.device_control(self.obj_id, "switch", '1')


    def turn_off(self):
        if self.data.get('oem_model') in HUMIDIFIER_OEM_MODEL:
            self.api.device_control(self.obj_id,"switch", '0')

        elif self.data.get('oem_model') in IRDEVICE_OEM_MODEL:


            with open(self.irdata_path) as f_obj:
                irdatas = json.load(f_obj)
            irCodes=irdatas["keys"]

            for code in irCodes:
                if code["fkey"]=="power_off":
                    endpointId = self.data.get('dsn')
                    self.api.ir_control(endpointId, "ifracmd_to_dev", code)
                else:
                    pass
            self.data["state"] = False
        else:
            self.api.device_control(self.obj_id, "switch1", '0')





from huihe.device.base import HuiheDevice


class HuiHeLight(HuiheDevice):


    def state(self):
        state = self.data.get('state')
        if state is None:
            return None
        return state


    def brightness(self):
        brightness = self.data.get('brightness')
        return brightness


    def _set_brightness(self, brightness):

        self.data['brightness'] = brightness


    def support_color(self):
        if self.data.get('color') is None:
            return False
        else:
            return True


    def support_color_temp(self):
        if self.dev_type=="0001-0201-0001":
            return False
        else:
            if self.data.get('color_temp') is None:

                return False
            else:
                return True


    def hs_color(self):
        if self.data.get('color') is None:
            return None
        else:
            work_mode = self.data.get('color_mode')
            if work_mode == 'colour':
                color = self.data.get('color')
                return color.get('hue'), color.get('saturation')
            else:
                return 0.0, 0.0


    def color_temp(self):
        if self.data.get('color_temp') is None:
            return None
        else:
            return self.data.get('color_temp')


    def min_color_temp(self):
        return 5500


    def max_color_temp(self):
        return 2800


    def turn_on(self):
        self.api.device_control(self.obj_id,"switch1", '1')


    def turn_off(self):
        self.api.device_control(self.obj_id,"switch1", '0')


    def set_brightness(self, brightness):
        """Set the brightness(0-255) of light."""
        value = int(brightness * 100 / 255)
        self.api.device_control(self.obj_id, 'brightness', value)


    def set_color(self, color):
        """Set the color of light."""
        hsv_color = {}
        hsv_color['hue'] = color[0]
        hsv_color['saturation'] = color[1]/100
        if (len(color) < 3):
            hsv_color['brightness'] = int(self.brightness()) / 255.0
        else:
            hsv_color['brightness'] = color[2]
        # color white
        if hsv_color['saturation'] == 0:
            hsv_color['hue'] = 0
        self.api.device_control(self.obj_id, 'colorSet', {'color': hsv_color})


    def set_color_temp(self, color_temp):
        self.api.device_control(self.obj_id, 'CCT', color_temp)



import json
import requests
# from requests import request
import time
import gzip
from huihe.device.get_huihe_device import get_huihe_device
import datetime

AYLA_DEVICE_SERVER = "ads-field.aylanetworks.com"  # 美国开发环境
APPID="huihe-d70b5148-field-us-id"
APPSECRET="huihe-d70b5148-field-us-orxaM7xo-jcuYLzvMKNwofCv9NQ"
TUYACLOUDURL = "https://px1.tuya{}.com"
DEFAULTREGION = 'us'

REFRESHTIME = 60 * 60 * 12

from  huihe.device.log import logger_obj


class HuiHeSession:

    username = ''
    password = ''
    countryCode = ''
    bizType = ''
    accessToken = ''
    refreshToken = ''
    expireTime = 0
    devices = []
    test={}
    region = DEFAULTREGION


SESSION = HuiHeSession()


class iFutureHomeApi:


    def init(self, username, password,hass, bizType=''):
        print('进入 iFutureHomeApi init')
        SESSION.username = username
        SESSION.password = password
        SESSION.bizType = bizType
        self.hass=hass
        self.requests=requests
        if username is None or password is None:
            return None
        else:
            self.get_access_token()
            self.discover_devices()
            return SESSION.devices


    def get_access_token(self):
        print('进入 get_access_token ')
        url = "https://" + AYLA_DEVICE_SERVER + "/users/sign_in.json"
        headers = {'Content-Type': "application/json"}
        data = {"user":{"email": SESSION.username,
                     "password" : SESSION.password,
                    "application": { "app_id": APPID,
                                     "app_secret": APPSECRET}}}

        try:
            # requests.packages.urllib3.disable_warnings()
            # import requests
            # response = self.hass.add_job（self.requests.request("POST", url, verify=False, data=json.dumps(data), headers=headers,timeout=6)）
            # response = requests.POST(url, verify=False, data=json.dumps(data), headers=headers,timeout=6)
            response = requests.request("POST", url, verify=False, data=json.dumps(data), headers=headers,timeout=6)
            jsonBody = json.loads(response.text)
            response_json = response.json()
            SESSION.test=jsonBody

            if response_json.get('responseStatus') == 'error':
                message = response_json.get('errorMsg')
                if message == 'error':
                    raise iFutureHomeAPIException("get access token failed")
                else:
                    raise iFutureHomeAPIException(message)

            SESSION.accessToken = response_json.get('access_token')
            SESSION.refreshToken = response_json.get('refresh_token')
            SESSION.expireTime = int(time.time()) + response_json.get('expires_in')
            # print("SESSION.refreshToken",SESSION.refreshToken)
            # print("SESSION.accessToken",SESSION.accessToken)
        except e:
            print(e)
            raise iFutureHomeAPIException("get access token failed")


    def check_access_token(self):
        if SESSION.username == '' or SESSION.password == '':
            raise iFutureHomeAPIException("can not find username or password")
            # return
        if SESSION.accessToken == '' or SESSION.refreshToken == '':
            self.get_access_token()
        elif SESSION.expireTime <= REFRESHTIME + int(time.time()):
            self.refresh_access_token()


    def refresh_access_token(self):
        headers = {
            'Content-Type': "application/json"
        }
        data={"user" : {"refresh_token" : SESSION.refreshToken}}
        url = "https://" + AYLA_DEVICE_SERVER + ":443/users/refresh_token.json"
        try:
            requests.packages.urllib3.disable_warnings()
            response = requests.request("POST", url, verify=False, data=json.dumps(data), headers=headers,timeout=6)
            jsonBody = json.loads(response.text)
            response_json = response.json()
        except:
            raise iFutureHomeAPIException('refresh token failed')

        if response_json.get('responseStatus') == 'error':
            raise iFutureHomeAPIException('refresh token failed')

        SESSION.accessToken = response_json.get('access_token')
        SESSION.refreshToken = response_json.get('refresh_token')
        SESSION.expireTime = int(time.time()) + response_json.get('expires_in')


    def poll_devices_update(self):
        self.check_access_token()
        return self.discover_devices()


    def discover_devices(self):
        body, code = self.get_alldevice()
        if code == 200 or code== 201 and len(body) != 0:
            logger_obj.debug("discover devices-get_alldevice success, code is ：  %s" + str(code))
            SESSION.devices = []
            for device in body:
                SESSION.devices.extend(get_huihe_device(self.hass,device, self))
            return body
        return None


    def get_IR_device(self,deviceId):
        headers = {
            'Authorization': 'Bearer ' +  str(SESSION.accessToken),
            'Content-Type': 'application/json',
        }
        url = "https://api11.ifuturehome.com.cn/pro/v1/devices/"+deviceId+"/subdevices"
        s = requests.session()
        s.keep_alive = False
        logger_obj.debug("get IR device url, url is：  %s" + str(url))
        logger_obj.debug("get IR device headers, headers is：  %s" + str(headers))
        requests.packages.urllib3.disable_warnings()
        body = []
        code=""
        try:
            response = requests.request("GET", url, verify=False, headers=headers,timeout=6)

            code = int(response.status_code)
            logger_obj.debug("get IR device code, code is：  %s"+ str(code))
            if code == 200 or code == 201:
                logger_obj.debug("get IR device success, code is：  %s"+ str(response.status_code))
                jsonBody = json.loads(response.text)
                body=jsonBody["body"]
            else:
                logger_obj.warning("get IR device error, error code is" + str(response.status_code))
                pass
        except Exception as err:
            logger_obj.warning("get IR device error ,Unexpected error : "+ str( err))
            pass

        return body, code


    def get_devices_by_type(self, dev_type):
        device_list = []
        for device in SESSION.devices:
            if device.dev_type() == dev_type:
                device_list.append(device)


    def get_all_devices(self):
        return SESSION.devices


    def get_device_by_id(self, dev_id):
        print("SESSION.devices:",SESSION.devices)
        for device in SESSION.devices:
            if device.object_id() == dev_id:
                return device
        return None


    def device_control(self,endpointId, propertyName,  value, param=None):
        if param is None:
            param = {}
        nowTime = datetime.datetime.now()
        logger_obj.warning("beging control device time is"+str(nowTime))
        response, code = self.control_request(endpointId, propertyName,value, param)
        if code == 200 or code==201:
            logger_obj.warning("device control success, code is" + str(code))
            nowTime = datetime.datetime.now()
            logger_obj.warning("finish control device time is"+str(nowTime))
            success = True
        else:
            logger_obj.warning("device control fail, code is" + str(code))
            nowTime = datetime.datetime.now()
            logger_obj.warning("finish control device time is"+str(nowTime))
            success = False
        return success,response


    def ir_control(self, endpointId, propertyName, value, param=None):

        codeType={
            'format': 1,
            'fre': 37900,
            'irdata_id': 11272,
            'keys': [
                {
                    'dcode': None,
                    'exts': None,
                    'fid': 0,
                    'fkey': 'power_off',
                    'fname': '',
                    'format': 0,
                    'scode': None,
                    'pulse': value["pulse"]
                }
            ],
            'rid': 1,
            'type': 2
        }

        jsonBody = json.dumps(codeType)  # 转成 JSON字符串
        value = jsonBody.encode()  # 编码
        value = gzip.compress(value)  # 压缩
        datapointValue = value.hex()  #转16进制
        if param is None:
            param = {}
        nowTime = datetime.datetime.now()
        logger_obj.warning("beging control ir device time is"+str(nowTime))
        response, code = self.control_request(endpointId, propertyName, datapointValue, param)
        if code == 200 or code == 201:
            logger_obj.debug("ir control success, code is：  %s"+ str(code))
            nowTime = datetime.datetime.now()
            logger_obj.debug("finish control ir device time is：  %s"+str(nowTime))
            success = True
        else:
            logger_obj.warning("ir control fail, code is：  %s"+ str(code))
            nowTime = datetime.datetime.now()
            logger_obj.warning("finish control ir device time is：  %s"+str(nowTime))
            success = False
        return success, response


    def retrieves_properties(self,deviceId):
        headers = {
            'Authorization': 'Bearer ' + str(SESSION.accessToken),
            'Content-Type': 'application/json',
        }
        url = "https://" + AYLA_DEVICE_SERVER + ":443/apiv1/dsns/"+deviceId+"/properties.json"
        requests.packages.urllib3.disable_warnings()
        nowTime = datetime.datetime.now()
        logger_obj.debug("beging retrieves_properties time is" + str(nowTime))
        propertyData = []
        try:
            response = requests.request("GET", url, verify=False, headers=headers,timeout=6)

            code = int(response.status_code)
            if code==200 or code==201:
                logger_obj.debug("retrieves_properties success, code is：  %s" + str(response.status_code))
                nowTime = datetime.datetime.now()
                logger_obj.debug("finish retrieves_properties device time is：  %s" + str(nowTime))
                jsonBody = json.loads(response.text)
                for property in jsonBody:
                    dict = {}
                    property = property["property"]
                    if property["value"] != None:
                        if property["name"] == "switch1":
                            if int(property["value"]) == 0:
                                dict["state"]=False
                            else:
                                dict["state"] =True
                            propertyData.append(dict)

                        elif property["name"] == "switch":
                            if int(property["value"]) == 0:
                                dict["state"]=False
                            else:
                                dict["state"] =True
                            propertyData.append(dict)

                        elif property["name"] == "brightness":
                            dict["brightness"] = int(property["value"]* 255 / 100)
                            propertyData.append(dict)

                        elif property["name"] == "CCT":
                                dict["color_temp"] = int(property["value"])
                                propertyData.append(dict)

                        elif property["name"] == "workmode":
                            dict["workmode"] = int(property["value"])
                            propertyData.append(dict)

                        elif property["name"] == "humi":
                            dict["target_humidity"] = int(property["value"])
                            propertyData.append(dict)

                        elif property["name"] == "realhumi":
                            dict["current_humidity"] = int(property["value"])
                            propertyData.append(dict)
                        elif property["name"] == "mist":
                            dict["mist"] = int(property["value"])
                            propertyData.append(dict)
                        else:
                            pass
                    else:
                        pass
            else:
                logger_obj.warning("retrieves device properties error, error code is：  %s" +str(response.status_code))
                nowTime = datetime.datetime.now()
                logger_obj.warning("finish retrieves_properties device time is：  %s"+ str(nowTime))
        except Exception as err:
            logger_obj.warning("retrieves device properties error ,Unexpected error : "+ str( err))
            pass
        return  propertyData


    def get_single_device(self,deviceId):
        headers = {
            'Authorization': 'Bearer ' + str(SESSION.accessToken),
            'Content-Type': 'application/json',
        }
        url = "https://" + AYLA_DEVICE_SERVER + ":443/apiv1/dsns/"+deviceId+".json"
        requests.packages.urllib3.disable_warnings()
        product_name = None
        connection_status=True
        try:
            response = requests.request("GET", url, verify=False, headers=headers,timeout=6)
            code = int(response.status_code)
            if code==200 or code==201:
                logger_obj.debug("get_single_device success, code is" + str(response.status_code))
                jsonBody = json.loads(response.text)
                product_name=jsonBody["device"]["product_name"]
                connection_status=jsonBody["device"]["connection_status"]
            else:
                logger_obj.warning("get_single_device error, error code is " +str(response.status_code))
                pass
        except Exception as err:
            logger_obj.warning("get_single_device error,Unexpected error : "+ str( err))
            pass
        # print("product_name,connection_status:",product_name,connection_status)
        return  product_name,connection_status


    def get_alldevice(self):

        headers = {
            'Authorization': 'Bearer ' +  str(SESSION.accessToken),
            'Content-Type': 'application/json',
        }
        url = "https://" + AYLA_DEVICE_SERVER + ":443/apiv1/devices.json"
        requests.packages.urllib3.disable_warnings()
        jsonBody=[]
        code=""
        try:
            response = requests.request("GET", url, verify=False, headers=headers,timeout=6)

            jsonBody = []
            code = int(response.status_code)
            if code == 200 or code == 201:
                logger_obj.debug("get all device success, code is ：  %s" + str(response.status_code))
                jsonBody = json.loads(response.text)
            else:
                logger_obj.warning("get all device fail, code is ：  %s" + str(response.status_code))
                pass
        except Exception as err:
            logger_obj.warning("get all device  error,Unexpected error : "+ str( err))

            response = requests.request("GET", url, verify=False, headers=headers, timeout=6)

            jsonBody = []
            code = int(response.status_code)
            if code == 200 or code == 201:
                logger_obj.debug("retry get all device success, code is ：  %s" + str(response.status_code))
                jsonBody = json.loads(response.text)
            else:
                logger_obj.warning("retry get all device fail, code is ：  %s" + str(response.status_code))
                pass

            pass

        return jsonBody, code

    def getACStateInfo(self, deviceId, subdeviceName):
        headers = {
            'Authorization': 'auth_token ' +  str(SESSION.accessToken),
            'Content-Type': 'application/json',
        }
        url = "https://api11.ifuturehome.com.cn/pro/v1/smartvoice/get_ac_state?device_id=" + deviceId + "&subdevice_name=" + subdeviceName
        requests.packages.urllib3.disable_warnings()
        code=""
        dict = {}
        try:
            response = requests.request("GET", url, verify=False, headers=headers,timeout=6)
            body = []
            code = int(response.status_code)

            if code == 200 or code == 201:
                logger_obj.debug("get AC State Info success, code is ：  %s" + str(response.status_code))
                jsonBody = json.loads(response.text)
                jsonBody = jsonBody["body"]
                dict["state"] = jsonBody["curPowerState"]
                dict["mode_list"] = jsonBody["mode_list"]
            else:
                logger_obj.warning("get AC State Info error, error code is：  %s" + str(response.status_code))
                pass
        except Exception as err:
            logger_obj.warning("get AC State Info  error,Unexpected error : "+ str( err))
            pass

        return dict, code


    def getIrCode(self,irdata_id):
        irdatas=""
        headers = {
            'Authorization': 'auth_token ' +  str(SESSION.accessToken),
            'Content-Type': 'application/json',
        }
        url = "https://api11.ifuturehome.com.cn/pro/v1/irdatas/" + str(irdata_id)
        requests.packages.urllib3.disable_warnings()
        logger_obj.debug("get IR code url, url is：  %s" + str(url))
        logger_obj.debug("get IR code headers, headers is：  %s" + str(headers))
        code=""
        try:
            response = requests.request("GET", url, verify=False, headers=headers,timeout=10)
            code = int(response.status_code)
            if code == 200 or code == 201:
                logger_obj.debug("get Ir code success, code is ：  %s" + str(response.status_code))
                jsonBody = json.loads(response.text)
                irdatas = jsonBody["body"]
            else:
                logger_obj.warning("get Ir code  error, error code is：  %s" + str(response.status_code))
                pass
        except Exception as err:
            logger_obj.warning("get Ir code  error, Unexpected error : "+ str( err))
            pass
        return irdatas,code


    def control_request(self,endpointId, propertyName, value, payload={}):
        payload['accessToken'] =  str(SESSION.accessToken)
        headers = {
            'Authorization': 'Bearer ' +  str(SESSION.accessToken),
            'Content-Type': 'application/json',
        }
        url = "https://" + AYLA_DEVICE_SERVER + ":443/apiv1/dsns/" + endpointId + "/properties/" + propertyName + "/datapoints.json"

        postBody = {'datapoint': {'value': value}}
        data=json.dumps(postBody)
        requests.packages.urllib3.disable_warnings()
        jsonBody = ""
        code=""
        try:
            response = requests.request("POST", url, verify=False, data=data, headers=headers,timeout=6)
            code = int(response.status_code)
            if int(code)!=200 and int(code)!=201:

                logger_obj.warning("control device error, error code is ：  %s" +str(code))
                pass

            else:
                logger_obj.debug("control device success, code is ：  %s" + str(code))
                jsonBody = json.loads(response.text)
        except Exception as err:
            logger_obj.warning("control device error, Unexpected error : "+ str( err))
            pass

        return jsonBody, code


    def changeChannel(self,type, subdeviceName, channelValue, endpointId, propertyName):
        headers = {
            'Authorization': 'auth_token ' +  str(SESSION.accessToken),
            'Content-Type': 'application/json',
        }
        if type=="name":
            url = "https://api11.ifuturehome.com.cn/pro/v1/smartvoice/irdata_by_channel_number?device_id=" + endpointId + "&subdevice_name=" + subdeviceName + "&channel_name=" + channelValue

        elif type=="number":
            url = "https://api11.ifuturehome.com.cn/pro/v1/smartvoice/irdata_by_channel_number?device_id=" + endpointId + "&subdevice_name=" + subdeviceName + "&channel_number=" + str(channelValue)
        logger_obj.debug("control AC Device url, url is：  %s" + str(url))
        requests.packages.urllib3.disable_warnings()
        dict = {}
        code=""
        try:
            response = requests.request("GET", url, verify=False, headers=headers,timeout=6)
            body = []
            code = int(response.status_code)

            if code == 200 or code == 201:
                jsonBody = json.loads(response.text)  # 将已编码的 JSON 字符串解码为 Python 对象
                if int(jsonBody["statusCode"]) != 200 or int(jsonBody["statusCode"]) != 201:  # 提取body
                    logger_obj.warning("get change Channel  fail, jsonBody is " + str(jsonBody))
                    pass
                else:
                    jsonBody = jsonBody["body"]  # 提取body
                    jsonBody = json.dumps(jsonBody)  # 转成 JSON字符串
                    value = jsonBody.encode()  # 编码
                    value = gzip.compress(value)  # 压缩

                    datapointValue = value.hex()
                    param = {}
                    response, code = self.control_request(endpointId, propertyName, datapointValue, param)
                    if code == 200 or code == 201:
                        logger_obj.debug("change Channel success, code is：  %s" + str(code))
                        pass
                    else:
                        logger_obj.warning("change Channel fail, code is ：  %s" + str(code))
                        pass
            else:
                pass
        except Exception as err:
            logger_obj.warning("change Channel erro, Unexpected error : " + str(err))
            pass

        return dict, code


    def controlACDevice(self, irEndpointId, subdeviceName, keyID, endpointId, propertyName):

        headers = {
            'Authorization': 'auth_token ' +  str(SESSION.accessToken),
            'Content-Type': 'application/json',
        }
        url = "https://api11.ifuturehome.com.cn/pro/v1/smartvoice/control_ac?device_id=" + endpointId + "&subdevice_name=" + subdeviceName + "&ac_param=" + keyID

        requests.packages.urllib3.disable_warnings()
        dict = {}
        code=""
        try:
            response = requests.request("GET", url, verify=False, headers=headers,timeout=6)
            body = []
            code = int(response.status_code)

            if code == 200 or code == 201:
                jsonBody = json.loads(response.text)    #将已编码的 JSON 字符串解码为 Python 对象
                logger_obj.debug("control AC Device statusCode, statusCode is：  %s" + str(jsonBody["statusCode"]))
                if int(jsonBody["statusCode"]) == 200 or int(jsonBody["statusCode"]) == 201:  # 提取body
                    logger_obj.debug("get control AC Device success, jsonBody is ：  %s" + str(jsonBody))
                    jsonBody = jsonBody["body"]  # 提取body
                    jsonBody = json.dumps(jsonBody)  # 转成 JSON字符串
                    value = jsonBody.encode()  # 编码
                    value = gzip.compress(value)  # 压缩

                    datapointValue = value.hex()
                    param = {}
                    response, code = self.control_request(endpointId, propertyName, datapointValue, param)
                    if code == 200 or code == 201:
                        logger_obj.debug("control AC Device success, code is ：  %s" + str(code))
                        pass
                    else:
                        logger_obj.warning("control AC Device fail, code is ：  %s" + str(code))

                        pass
                else:

                    logger_obj.warning("get control AC Device fail, jsonBody is ：  %s"+ str(jsonBody))
                    pass
            else:
                pass
        except Exception as err:
            logger_obj.warning("get control AC Device fail, Unexpected error : "+ str(err))
            pass
        return dict, code


    def controlIRDevice(self, irEndpointId, subdeviceName, keyID, endpointId, propertyName):
        headers = {
            'Authorization': 'auth_token ' +  str(SESSION.accessToken),
            'Content-Type': 'application/json',
        }
        url = "https://api11.ifuturehome.com.cn/pro/v1/smartvoice/irdata_by_fid?device_id=" + endpointId + "&subdevice_name=" + subdeviceName + "&fid=" + keyID
        requests.packages.urllib3.disable_warnings()
        dict = {}
        code=""
        try:
            response = requests.request("GET", url, verify=False, headers=headers,timeout=6)
            body = []
            code = int(response.status_code)

            if code == 200 or code == 201:
                jsonBody = json.loads(response.text)    #将已编码的 JSON 字符串解码为 Python 对象

                if int(jsonBody["statusCode"])!=200 and int(jsonBody["statusCode"]) != 201: #提取body
                    logger_obj.warning("control IR Device fail, jsonBody is " + str(jsonBody))

                    pass
                else:
                    jsonBody = jsonBody["body"]  # 提取body
                    jsonBody=json.dumps(jsonBody) #转成 JSON字符串
                    value = jsonBody.encode()  #编码
                    value = gzip.compress(value)   #压缩
                    datapointValue = value.hex()
                    param = {}
                    response, code = self.control_request(endpointId, propertyName, datapointValue, param)
                    if code == 200 or code == 201:
                        logger_obj.debug("control IR Device success, code is ：  %s" +str(code))
                        pass
                    else:
                        logger_obj.warning("control IR Device fail, code is ：  %s" +str(code))
                        pass
            else:
                pass
        except Exception as err:
            logger_obj.warning("control IR Device fail, Unexpected error : "+ str(err))
            pass
        return dict, code


    def controlIRDeviceExceptAC(self, irEndpointId, subdeviceName, keyID, endpointId, propertyName):
        headers = {
            'Authorization': 'auth_token ' +  str(SESSION.accessToken),
            'Content-Type': 'application/json',
        }
        url="https://api11.ifuturehome.com.cn/pro/v1/smartvoice/irdata_by_fid?device_id=" + endpointId + "subdevice_name=" + subdeviceName + "fid=" + keyID
        requests.packages.urllib3.disable_warnings()
        dict = {}
        code=""
        try:
            response = requests.request("GET", url, verify=False, headers=headers,timeout=6)
            body = []
            code = int(response.status_code)

            if code == 200 or code == 201:
                jsonBody = json.loads(response.text)    #将已编码的 JSON 字符串解码为 Python 对象
                if int(jsonBody["statusCode"]) != 200 or int(jsonBody["statusCode"]) != 201:  # 提取body
                    logger_obj.warning("control IR Device Except AC fail, jsonBody is " + str(jsonBody))
                    pass
                else:
                    logger_obj.info("control IR Device Except AC success, jsonBody is " + str(jsonBody))
                    jsonBody = jsonBody["body"] #提取body
                    jsonBody=json.dumps(jsonBody) #转成 JSON字符串
                    value = jsonBody.encode()  #编码
                    value = gzip.compress(value)   #压缩
                    datapointValue = value.hex()
                    param = {}
                    response, code = self.control_request(endpointId, propertyName, datapointValue, param)
                    if code == 200 or code == 201:

                        logger_obj.debug("control IR Device Except AC success, code is ：  %s" +str(code))
                        pass
                    else:
                        logger_obj.warning("control IR Device Except AC fail, code is：  %s"+ str(code))
                        pass
            else:
                pass
        except Exception as err:
            logger_obj.warning("control IR Device Except AC fail, Unexpected error : "+ str(err))
            pass

        return dict, code



class iFutureHomeAPIException(Exception):
    pass





from huihe.device.HuiHeClimate import HuiHeClimate
from huihe.device.HuiHeMediaPlayer import HuiHeMediaPlayer
import copy
def HuiHeIR(hass,data, api):
    devices = []
    deviceId=data["dsn"]

    body,code=api.get_IR_device(deviceId)
    for device in body:
        DATA = ""
        DATA = copy.deepcopy(data)
        DATA["subdevice_name"]=device["subdevice_name"]
        DATA["subdevice_id"] = device["subdevice_id"]
        DATA["device_id"] = device["device_id"]
        DATA["irdata_id"] = device["irdata_id"]
        if device["subdevice_typeID"]==2:

            DATA["dev_type"] = "tv"
            devices.append(HuiHeMediaPlayer(hass,DATA, api))
        elif device["subdevice_typeID"] == 5:

            DATA["dev_type"] = "ac"
            devices.append(HuiHeClimate(hass,DATA, api))
        elif device["subdevice_typeID"] == 8:
            pass
        elif device["subdevice_typeID"] == 10:
            pass

    return devices



from huihe.device.base import HuiheDevice

class HuiHeSwitch(HuiheDevice):


    def state(self):
        state = self.data.get('state')
        if state is None:
            return None
        return state


    def turn_on(self):
        self.api.device_control(self.obj_id,"switch1", '1')


    def turn_off(self):
        self.api.device_control(self.obj_id,"switch1", '0')





from huihe.device.base import HuiheDevice
import json
from huihe.device.mediaPlayerConst import (
    ATTR_APP_ID, ATTR_APP_NAME, ATTR_INPUT_SOURCE, ATTR_INPUT_SOURCE_LIST,
    ATTR_MEDIA_ALBUM_ARTIST, ATTR_MEDIA_ALBUM_NAME, ATTR_MEDIA_ARTIST,
    ATTR_MEDIA_CHANNEL, ATTR_MEDIA_CONTENT_ID, ATTR_MEDIA_CONTENT_TYPE,
    ATTR_MEDIA_DURATION, ATTR_MEDIA_EPISODE,
    ATTR_MEDIA_PLAYLIST, ATTR_MEDIA_POSITION, ATTR_MEDIA_POSITION_UPDATED_AT,
    ATTR_MEDIA_SEASON, ATTR_MEDIA_SERIES_TITLE,
    ATTR_MEDIA_SHUFFLE, ATTR_MEDIA_TITLE, ATTR_MEDIA_TRACK,
    ATTR_MEDIA_VOLUME_LEVEL, ATTR_MEDIA_VOLUME_MUTED, ATTR_SOUND_MODE,
    ATTR_SOUND_MODE_LIST, SUPPORT_NEXT_TRACK, SUPPORT_PREVIOUS_TRACK, SUPPORT_TURN_OFF,
    SUPPORT_TURN_ON, SUPPORT_VOLUME_MUTE, SUPPORT_VOLUME_SET,
    SUPPORT_VOLUME_STEP)


SUPPORT_TV = SUPPORT_VOLUME_STEP | SUPPORT_TURN_ON | SUPPORT_VOLUME_SET | \
             SUPPORT_TURN_OFF | SUPPORT_PREVIOUS_TRACK | SUPPORT_NEXT_TRACK | SUPPORT_VOLUME_MUTE

ATTR_TO_PROPERTY = [
    ATTR_MEDIA_VOLUME_LEVEL,
    ATTR_MEDIA_VOLUME_MUTED,
    ATTR_MEDIA_CONTENT_ID,
    ATTR_MEDIA_CONTENT_TYPE,
    ATTR_MEDIA_DURATION,
    ATTR_MEDIA_POSITION,
    ATTR_MEDIA_POSITION_UPDATED_AT,
    ATTR_MEDIA_TITLE,
    ATTR_MEDIA_ARTIST,
    ATTR_MEDIA_ALBUM_NAME,
    ATTR_MEDIA_ALBUM_ARTIST,
    ATTR_MEDIA_TRACK,
    ATTR_MEDIA_SERIES_TITLE,
    ATTR_MEDIA_SEASON,
    ATTR_MEDIA_EPISODE,
    ATTR_MEDIA_CHANNEL,
    ATTR_MEDIA_PLAYLIST,
    ATTR_APP_ID,
    ATTR_APP_NAME,
    ATTR_INPUT_SOURCE,
    ATTR_INPUT_SOURCE_LIST,
    ATTR_SOUND_MODE,
    ATTR_SOUND_MODE_LIST,
    ATTR_MEDIA_SHUFFLE,
]

CACHE_IMAGES = 'images'
CACHE_MAXSIZE = 'maxsize'
CACHE_LOCK = 'lock'
FILENAME=".homeassistant/irdata_id.json"

class HuiHeMediaPlayer(HuiheDevice):


    def state(self):
        """State of the player."""
        return None


    def name(self):
        """Return the display name of this TV."""
        subdeviceName = self.data.get('subdevice_name')
        return subdeviceName


    def volume_level(self):
        """Volume level of the media player (0..1)."""
        return None


    def is_volume_muted(self):
        """Boolean if volume is currently muted."""
        return None


    def media_content_type(self):
        """Content type of current playing media."""
        return "MEDIA_TYPE_CHANNEL"
        # return None


    def media_channel(self):
        """Channel currently playing."""
        return None


    def supported_features(self):
        """Flag media player features that are supported."""
        return SUPPORT_TV


    def power(self):
        """Turn the media player off."""

        with open(FILENAME) as f_obj:
            irdatas = json.load(f_obj)
        irCodes = irdatas["keys"]

        for code in irCodes:
            if code["fkey"] == "power":
                endpointId = self.data.get('dsn')
                self.api.ir_control(endpointId, "ifracmd_to_dev", code)
            else:
                pass


    def volume_up(self):
        """Increase volume by one."""


        with open(FILENAME) as f_obj:
            irdatas = json.load(f_obj)
        irCodes = irdatas["keys"]

        for code in irCodes:
            if code["fkey"] == "volume_up":
                endpointId = self.data.get('dsn')
                self.api.ir_control(endpointId, "ifracmd_to_dev", code)
            else:
                pass


    def volume_down(self):
        """Decrease volume by one."""
        with open(FILENAME) as f_obj:
            irdatas = json.load(f_obj)
        irCodes = irdatas["keys"]

        for code in irCodes:
            if code["fkey"] == "volume_down":
                endpointId = self.data.get('dsn')
                self.api.ir_control(endpointId, "ifracmd_to_dev", code)
            else:
                pass


    def mute_volume(self, mute):
        """Mute the volume."""
        with open(FILENAME) as f_obj:
            irdatas = json.load(f_obj)
        irCodes = irdatas["keys"]

        for code in irCodes:
            if code["fkey"] == "mute_volume":
                endpointId = self.data.get('dsn')
                self.api.ir_control(endpointId, "ifracmd_to_dev", code)
            else:
                pass


    def set_volume_level(self, volume):
        """Set volume level, range 0..1."""
        raise NotImplementedError()


    def media_previous_track(self):
        """Send previous track command."""

        with open(FILENAME) as f_obj:
            irdatas = json.load(f_obj)
        irCodes = irdatas["keys"]

        for code in irCodes:
            if code["fkey"] == "channel_down":
                endpointId = self.data.get('dsn')
                self.api.ir_control(endpointId, "ifracmd_to_dev", code)
            else:
                pass

        keyID = '43'


    def media_next_track(self):
        """Send next track command."""
        keyID = '44'
        with open(FILENAME) as f_obj:
            irdatas = json.load(f_obj)
        irCodes = irdatas["keys"]

        for code in irCodes:
            if code["fkey"] == "channel_up":
                endpointId = self.data.get('dsn')
                self.api.ir_control(endpointId, "ifracmd_to_dev", code)
            else:
                pass


    def select_channel(self, command, type):
        subdeviceName = self.data.get('subdevice_name')
        endpointId = self.data.get('dsn')
        self.api.changeChannel(type, subdeviceName, command, endpointId, 'ifracmd_to_dev')


    def media_seek(self, position):
        """Send seek command."""
        raise NotImplementedError()


    def select_source(self, source):
        """Select input source."""
        raise NotImplementedError()


    def select_sound_mode(self, sound_mode):
        """Select sound mode."""
        raise NotImplementedError()




